/*
                                 mondesi.js

Relies heavily on functionality provided by WZ_JSGRAPHICS.JS (thanks Walter).

Plotting routines assume a 300w x 100h graph (yep, q&d).


COPYRIGHT
---------
Copyright (C) 2008-2013 Mark G.Daniel
This program, comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it under the
conditions of the GNU GENERAL PUBLIC LICENSE, version 3, or any later version.
http://www.gnu.org/licenses/gpl.txt


VERSION
-------
31-MAR-2013  MGD  per-process monitoring
22-JAN-2013  MGD  finally discovered what garbles printable WZ graphics on
                    MSIE (or at least how to prevent it) - reset "jg_fast"
06-JAN-2013  MGD  refinement (perhaps rework is closer)
13-OCT-2010  MGD  refinement
08-JUL-2010  MGD  add WebSocket support
29-NOV-2008  MGD  initial
*/

// versions of MONDESI.EXE this JavaScript is compatible with
var mdsCompatibleVersions = new Array ("4.0.0"); 

// note: many storage items are instantiated by MONDESI.C

var mdsBarCol1 = "LimeGreen",
    mdsBarCol2 = "Chocolate",
    mdsBarColAve = "Yellow",
    mdsHistCol1 = "DarkGreen",
    mdsHistCol2 = "Sienna",
    mdsHistColAve = "GoldenRod",
    mdsObviousBtn = "Gold";

var mdsData = new Array();

var mdsAlertSecs = 90,  // ninety seconds
    mdsClusterVisible = false,
    mdsDiskMount = 0,
    mdsDiskVisible = false,
    mdsNodeHistVisible = false,
    mdsMSIEversion = 0,
    mdsPauseInProgress = false,
    mdsPauseStopTimer = null,
    mdsPauseStopSecs = 300,   // five minutes
    mdsPerProcPid = 0,
    mdsPerProcVisible = false,
    mdsPerProcHistVisible = false,
    mdsPrintInProgress = false,
    mdsPrintTimeout = 30,  // 30 seconds
    mdsRunning = false,
    mdsScaleTimer = null,
    mdsScaleSecs = 300,  // five minutes
    mdsTimeStamp = 0,
    mdsTopProcVisible = false,
    mdsUpdateDisplay = false,
    mdsUpdatePause = false,
    mdsVersionNBG = false,
    mdsWzPrintable = false;

var mdsAlertSpan = '<span style="font-size:140%;font-weight:bold;' +
                   'margin:5px;float:right;color:red;">';
var mdsAlertEndSpan = '</span>';

var mdsAlertText = mdsAlertSpan + 'STALLED' + mdsAlertEndSpan;

var mdsPauseStopText = mdsAlertSpan + 'STOPPED' + mdsAlertEndSpan;

var mdsProcTable = '<table id="proc_table" class="proc_table">' +
                   '<tr><th>CPU</th><th>S+E+K</th><th>FLTS</th><th>BIO</th>' +
                   '<th>DIO</th></tr>';
var mdsProcEndTable = '</table>';

var mdsPerProcTable = '<table id="per_proc_table" class="proc_table" ' +
                      'onclick="mdsPerProcClick(0)">' +
                      '<tr><td></td><td></td><td></td><td></td><td></td></tr>';
var mdsPerProcEndTable = '</table>';

var mdsIncompVersion = 'JavaScript and executable are incompatible!\n' +
                       'Reload and/or clear browser cache.';

if (mdsCompatibleVersions.indexOf(mdsExeVersion) == -1) mdsVersionNBG = true;

// initial start
// -------------

function mdsInitialStart ()
{
   if (mdsVersionNBG)
   {
      alert(mdsIncompVersion);
      return;
   }

   if (navigator.appName == 'Microsoft Internet Explorer')
   {
      var ua = navigator.userAgent;
      var re  = new RegExp("MSIE ([0-9]{1,}[\.0-9]{0,})");
      // MSIE sacrifice speed for correct rendering
      if (re.exec(ua) != null) mdsMSIEversion = parseFloat(RegExp.$1);
   }

   if (mdsCluster) {
      var scsBtn = document.getElementById("scs_btn");
      scsBtn.disabled = false;
   }

   if (mdsDiskCount > 0) {
      var diskBtn = document.getElementById("disk_btn");
      diskBtn.disabled = false;
   }

   mdsStreamData (true);
   mdsSetDisplayHeight ();
   mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
   mdsRunning = mdsUpdateDisplay = true;
}

// the MonDeSi executable has (re)started
// --------------------------------------

function mdsExecStartup ()
{
   mdsProcDisplay();
   mdsPerProcDisplay();
}

// update the textual node information
// -----------------------------------

function mdsNodeNotes (text)
{
   if (!mdsUpdateDisplay) return;

   var nodeNotes = document.getElementById("node_notes");
   nodeNotes.innerHTML = null;
   nodeNotes.innerHTML = text;
   // each minute the alert gets recheduled if updating is occuring
   mdsResetAlert(true);
}

// (re)initialise item data
// ------------------------

function mdsDataInit (which,active)
{
   if (mdsData[which] == undefined)
   {
      mdsData[which] = new Object();
      mdsData[which].timer = null;
      mdsData[which].which = which;
      mdsData[which].title = which.substr(which.indexOf('_')+1).toUpperCase();
      mdsData[which].active = active;
      if (which.substr(0,5) == "node_")
         mdsData[which].bar_jsgr = new jsGraphics("bar_"+which);
      mdsData[which].hist_jsgr = new jsGraphics("hist_"+which);
   }
   else
   {
      delete mdsData[which].hist1;
      delete mdsData[which].hist2;
   }

   mdsData[which].ave = mdsData[which].max =
      mdsData[which].val1 = mdsData[which].val2 =
      mdsData[which].scale = 0;
   mdsData[which].fixed = false;
   mdsData[which].visible = false;
   mdsData[which].hist1 = new Array();
   mdsData[which].hist2 = new Array();

   if (which.substr(0,5) == "node_")
   {
      mdsBarGraph (which);
      mdsBarPaint (which);
   }

   mdsHistGraph (which);
   mdsHistPaint (which);
}

// set and repaint the title
// -------------------------

function mdsDataTitle (which,title)
{
   mdsData[which].title = title;
   mdsDatum(which);
}

// scale displays according to maximum datum
// -----------------------------------------

function mdsScale ()
{
   for (which in mdsData)
   {
      if (mdsData[which].fixed) continue;
      var hist1 = mdsData[which].hist1;

      var max = 0;
      for (tdata in hist1)
         if (hist1[tdata] > max) max = hist1[tdata];
      mdsData[which].max = mdsData[which].scale = max;
      mdsData[which].ave = 0;
      if (which.substr(0,5) == "node_") mdsBarPlot(which);
      mdsHistPlot(which);
   }

   clearTimeout(mdsScaleTimer);
   mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
}

// zero data, reset graphs
// -----------------------

function mdsZeroData ()
{
   for (which in mdsData)
   {
      mdsDataInit(which);
      mdsDatum(which,0);
   }
}

// record a datum and graph its value
// ----------------------------------
// (called by the data streaming from the MonDeSi executable)

function mdsDatum (which,scale,value1,value2)
{
   if (mdsData[which] == undefined) return;

   if (!mdsData[which].active) return;

   if (scale == undefined) scale = -1;
   if (value1 == undefined) value1 = -1;
   if (value2 == undefined) value2 = -1;

   // arrange a timed update (must be done before 'scale' is adjusted)
   var updateGraph = 'mdsDatum("'+which+'",'+scale+','+value1+','+value2+')';
   clearTimeout(mdsData[which].timer);
   mdsData[which].timer = setTimeout(updateGraph,mdsRefreshInterval*2000);

   if (value1 > mdsData[which].max) mdsData[which].max = value1;
   var max = mdsData[which].max;
   if (scale >= 0)
      mdsData[which].fixed = true;
   else
      scale = max;
   mdsData[which].scale = scale;

   // calculate an average for most data
   var average = 0;
   if (which != "node_mem" && which != "node_pgfl" && which != "node_prc")
   {
      var hist1 = mdsData[which].hist1;
      var count = 0;
      var total = 0;
      for (tdata in hist1)
      {
         count++;
         total += hist1[tdata];
      }
      if (count > 10) average = Math.round((total / count));
   }

   mdsData[which].val1 = value1;
   mdsData[which].val2 = value2;
   mdsData[which].ave = average;

   // record the datum in a sparse (js hash) array indexed by timestamp
   var tdata = Math.round(mdsTimeStamp).toString();
   if (value1 >= 0) mdsData[which].hist1[tdata] = value1;
   if (value2 >= 0) mdsData[which].hist2[tdata] = value2;

   if (which.substr(0,5) == "node_") mdsBarPlot(which);

   mdsHistPlot(which);
}

// set the process table
// ---------------------

function mdsMonProc (table)
{
   if (!mdsUpdateDisplay) return;

   var procTableDiv = document.getElementById("proc_table_div");
   procTableDiv.innerHTML = null;
   procTableDiv.innerHTML = mdsProcTable + table + mdsProcEndTable;
}

// now make all PIDs clickable
// ---------------------------

function mdsMonProcPid (table)
{
   if (!mdsUpdateDisplay) return;

   var table = document.getElementById("proc_table");
   for (var i = 0, row; row = table.rows[i]; i++) {
      for (var j = 0, col; col = row.cells[j]; j++) {
         var pid = col.innerHTML.substr(0,8);
         if (pid.length == 8) col.onclick = mdsPidClick;
      }  
   }
}

// response to a PID click
// -----------------------

function mdsPidClick (event)
{ 
   var pid = event.currentTarget.innerHTML.substr(0,8);
   mdsPerProcClick (pid);
   event.cancelBubble = true;  // MSIE
   if (event.stopPropagation) event.stopPropagation();
}

// prompt for an ad hoc PID
// ------------------------

function mdsPidPrompt (event)
{
   pid = prompt('Enter PID','');
   if (!isNaN(parseInt(pid,16))) mdsPerProcClick(pid);
   event.cancelBubble = true;  // MSIE
   if (event.stopPropagation) event.stopPropagation();
}

// per-process table
// -----------------

function mdsMonPerProc (table)
{
   if (!mdsUpdateDisplay) return;

   var perProcTableDiv = document.getElementById("per_proc_table_div");
   perProcTableDiv.innerHTML = null;
   perProcTableDiv.innerHTML = mdsPerProcTable + table + mdsPerProcEndTable;
}

// cancel any outstanding alert and optionally set a new one
// ---------------------------------------------------------

var mdsAlertTimer = null;

function mdsResetAlert (newAlert)
{
   // each minute the alert gets recheduled if updating is occuring
   clearTimeout(mdsAlertTimer);
   mdsAlertTimer = null;
   if (newAlert) mdsAlertTimer = setTimeout("mdsAlert()",mdsAlertSecs*1000);
}

// that the node has stopped being updated
// ---------------------------------------

function mdsAlert ()
{
   // the above timer has expired
   mdsAlertTimer = null;
   var nodeNotes = document.getElementById("node_notes");
   var notesText = nodeNotes.innerHTML;
   nodeNotes.innerHTML = null;
   nodeNotes.innerHTML = mdsAlertText + notesText;
}

// send a message from a child iframe to the parent
// ------------------------------------------------

function mdsToMsgChannel (forEval)
{
   // MSIE(6) and SeaMonkey(1.6) (at least) deny permission for these
   try
   {
      var win = parent.window;
      if (win != undefined && typeof win.postMessage == "function")
         // message posting browsers (HTML5)
         win.postMessage (forEval, "*");
      else
         // inter-frame kludge
         mdsToBackChannel (forEval);
   }
   catch (err)
   {
      // inter-frame kludge
      mdsToBackChannel (forEval);
   }
}

// message passing via hidden iframe (HTML4)
// -----------------------------------------

var mdsBackChanCount = 0;

function mdsToBackChannel (forEval)
{
   var url = mdsRefererHost + "/mondesi/-/js/bchan.html#" + forEval;
   // MSIE(6+7) gets confused removing then adding elements with identical IDs!
   var iframe = document.getElementById("bchan_iframe_"+mdsBackChanCount);
   if (iframe != undefined)
   {
      mdsBody.removeChild(iframe);
      delete iframe;
   }
   iframe = document.createElement("iframe");
   iframe.setAttribute("id","bchan_iframe_"+(mdsBackChanCount++));
   iframe.setAttribute("src",url);
   iframe.setAttribute("height",0);
   iframe.setAttribute("width",0);
   iframe.setAttribute("frameBorder",0);
   mdsBody.appendChild(iframe);
}

// stop/start the streaming data flow
// ----------------------------------

var mdsDataIframeCount = 0;
var mdsDataIframe = undefined;

function mdsStreamData (start)
{
   if (WASDwebSocket)
   {
      // close any existing connections
      webSocketClose();
      mdsResetAlert (false);
      if (!start) return;

      if (webSocketOpen())
      {
         mdsResetAlert (true);
         return;
      }
      // it's entirely possible the server but not browser supports WebSocket
      WASDwebSocket = false;
   }

   // via iframe and HTML push

   if (mdsDataIframe != undefined)
   {
      // MSIE (6+7) seems to need this to dissociate from the data stream
      mdsDataIframe.src = "/mondesi/-/js/nl.html";

      /* dispose of the iframe dropping the data stream */
      mdsBody.removeChild(mdsDataIframe);
      delete mdsDataIframe;
      mdsDataIframe = undefined;
      mdsResetAlert (false);
   }

   if (start)
   {
      var url = mdsScriptName + "?" +
                (mdsTopProcVisible ? "1" : "0") + "+" + mdsPerProcPid;
      mdsDataIframe = document.createElement("iframe");
      // MSIE(6+7) gets confused removing then adding elements with same IDs!
      mdsDataIframe.setAttribute("id","node_iframe_"+mdsDataIframeCount);
      mdsDataIframe.setAttribute("src",url);
      mdsDataIframe.setAttribute("height",0);
      mdsDataIframe.setAttribute("width",0);
      mdsDataIframe.setAttribute("frameBorder",0);
      mdsBody.appendChild(mdsDataIframe);
      mdsResetAlert (true);
   }
}

// cluster button has been toggled
// -------------------------------

function mdsClusterClick (setValue)
{
   var nodeScs = document.getElementById("node_scs");
   var histScs = document.getElementById("hist_node_scs");
   if (mdsCluster && setValue == undefined)
      mdsClusterVisible = !mdsClusterVisible;
   if (mdsClusterVisible) {
      nodeScs.style.display = "inline";
      // better as "table-row" but <MSIE8 won't accept it
      histScs.style.display = "";
   }
   else
      nodeScs.style.display = histScs.style.display = "none";
   mdsSetDisplayHeight();
   return true;
}

// disk button has been toggled
// ----------------------------

function mdsDiskClick (setValue)
{
   var nodeDisk = document.getElementById("node_disk_device");
   if (mdsDiskCount > 0 && setValue == undefined)
      mdsDiskVisible = !mdsDiskVisible;
   if (mdsDiskVisible)
      nodeDisk.style.display = "inline";
   else
      nodeDisk.style.display = "none";
   mdsSetDisplayHeight();
   return true;
}

// control use of the [proc] button
// --------------------------------

function mdsProcButton (top,per)
{
   document.getElementById("proc_btn").disabled = !top;
   if (!top) {
      mdsTopProcVisible = per = false;
      mdsProcDisplay();
      controlTopProc();
   }
   if (!per) {
      mdsPerProcVisible = false;
      mdsPerProcPid = 0;
      mdsPerProcDisplay();
      controlPerProc ();
   }
}

// top-process display
// -------------------

function mdsProcClick (setValue)
{
   if (setValue == undefined) {
      if (!document.getElementById("proc_btn").disabled)
         mdsTopProcVisible = !mdsTopProcVisible;
   }
   else
      mdsTopProcVisible = setValue;
   mdsProcDisplay();
   controlTopProc();
   return true;
}

// set display of top-processes
// ----------------------------

function mdsProcDisplay ()
{
   var procDiv = document.getElementById("proc_wrapt");
   var procTableDiv = document.getElementById("proc_table_div");
   if (mdsTopProcVisible) {
      procDiv.style.display = "block";
      procTableDiv.innerHTML = null;
      procTableDiv.innerHTML = mdsProcTable + mdsProcEndTable;
   }
   else
   if (procDiv) {
      procDiv.style.display = "none";
      procTableDiv.innerHTML = null;
      mdsPerProcVisible = mdsPerProcHistVisible = mdsPerProcPid = 0;
   }
   mdsPerProcDisplay ();
   mdsSetDisplayHeight();
}

// per-process display
// -------------------

function mdsPerProcClick (pid)
{
   mdsPerProcVisible = ((mdsPerProcPid = pid) != 0);
   if (!mdsPerProcVisible) mdsPerProcHistVisible = false;

   // zero the per-process data
   for (which in mdsData)
   {
      if (which.substr(0,5) == "node_") continue;
      mdsDataInit(which);
      mdsDatum(which,0);
   }

   mdsPerProcDisplay();
   mdsPerProcHistDisplay();
   controlPerProc ();
   return true;
}

// set display of per-process
// --------------------------

function mdsPerProcDisplay ()
{
   var perProcDiv = document.getElementById("per_proc_wrapt");
   var perProcTableDiv = document.getElementById("per_proc_table_div");
   if (mdsPerProcVisible) {
      perProcDiv.style.display = "block";
      perProcTableDiv.innerHTML = mdsPerProcTable + mdsPerProcEndTable;
   }
   else
   if (perProcDiv) {
      perProcDiv.style.display = "none";
      perProcTableDiv.innerHTML = null;
      perProcHistVisible = false;
   }
   mdsPerProcHistDisplay();
   mdsSetDisplayHeight();
}

// per-process history
// -------------------

function mdsPerProcHistClick (event)
{
   if (mdsPerProcHistVisible)
      mdsPerProcHistVisible = false;
   else
      mdsPerProcHistVisible = mdsPerProcVisible;
   mdsPerProcHistDisplay();
   event.cancelBubble = true;  // MSIE
   if (event.stopPropagation) event.stopPropagation();
   return true;
}

// set display of per-process history
// ----------------------------------

function mdsPerProcHistDisplay ()
{
   var perProcHistWrapt = document.getElementById("per_proc_hist_wrapt");
   if (mdsPerProcHistVisible)
      perProcHistWrapt.style.display = "block";
   else
   if (perProcHistWrapt)
      perProcHistWrapt.style.display = "none";
   mdsSetDisplayHeight();
}

// history button has been toggled
// -------------------------------

function mdsHistoryClick (setValue)
{
   var histGraph = document.getElementById("hist_node_graph");
   if (setValue == undefined) mdsNodeHistVisible = !mdsNodeHistVisible;
   if (mdsNodeHistVisible)
      histGraph.style.display = "inline";
   else
      histGraph.style.display = "none";
   mdsSetDisplayHeight();
   return true;
}

// set the display height
// ----------------------

function mdsSetDisplayHeight ()
{
   var nodeHeight = (mdsBarHeight * 9) + 20;
   var histHeight = ((mdsGraphHeight + 10) * 3);
   var perProcHistHeight = ((mdsGraphHeight + 10) * 2);

   if (mdsClusterVisible) {
      nodeHeight += mdsBarHeight * 3;
      histHeight += mdsGraphHeight + 10;
   }

   if (mdsDiskVisible) {
      nodeHeight += mdsBarHeight * mdsDiskCount;
      histHeight += (mdsGraphHeight + 10) * Math.floor(mdsDiskCount / 3);
      if (mdsDiskCount % 3) histHeight += (mdsGraphHeight + 10);
   }

   var displayHeight = nodeHeight;

   if (mdsTopProcVisible) {
      displayHeight += mdsProcHeight + 4;
      if (mdsPerProcVisible) {
         displayHeight += mdsPerProcHeight + 4;
         if (mdsPerProcHistVisible) {
            displayHeight += perProcHistHeight + 4;
            if (mdsNodeHistVisible) displayHeight += 26;
         }
      }
   }

   if (mdsNodeHistVisible) displayHeight += histHeight + 4;

   var remotePC = "mdsIframeHeight('" + mdsIframeId + "'," +
                  mdsNodeHistVisible + "," + displayHeight + ")";
   mdsToMsgChannel(remotePC);
}

// pause and release the display update
// ------------------------------------

var mdsPauseBgCol;

function mdsPauseClick ()
{ 
   var btn = document.getElementById("pause_btn");
   if (mdsRunning && !mdsPauseInProgress)
   {
      // pause the data updates
      mdsPauseBgCol = btn.style.backgroundColor;
      btn.style.backgroundColor = mdsObviousBtn;
      btn.value = "release";
      mdsResetAlert(false);
      clearTimeout(mdsScaleTimer);
      mdsScaleTimer = null;
      mdsPauseStopTimer = setTimeout("mdsPauseStop()",mdsPauseStopSecs*1000);
      // just in case the browser print dialogue is engaged whilst stopped
      mdsMakePrintable(true);
      mdsPauseInProgress = true;
      mdsUpdateDisplay = false;
   }
   else
   if (mdsRunning)
   {
      // recommence the data updates
      btn.style.backgroundColor = mdsPauseBgCol;
      btn.value = "pause";
      mdsResetAlert(true);
      mdsMakePrintable(false);
      mdsProcClick(mdsTopProcVisible);
      clearTimeout(mdsPauseStopTimer);
      mdsPauseStopTimer = null;
      mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
      mdsPauseInProgress = false;
      if (!mdsPrintInProgress) mdsUpdateDisplay = true;
   }
   else
   {
      // restart data collection and display
      btn.style.backgroundColor = mdsPauseBgCol;
      btn.value = "pause";
      mdsStreamData (true);
      mdsZeroData();
      if (mdsPrintInProgress) 
         mdsPrintClick();
      else
         mdsMakePrintable(false);
      mdsProcClick(mdsTopProcVisible);
      mdsScaleTimer = setTimeout("mdsScale()",mdsScaleSecs*1000);
      mdsPauseInProgress = false;
      mdsRunning = mdsUpdateDisplay = true;
   }
   return true;
}

// timer-driven stop
// -----------------

function mdsPauseStop ()
{
   // been paused for long enough!
   mdsRunning = mdsUpdateDisplay = false;
   mdsPauseStopTimer = null;
   clearTimeout(mdsScaleTimer);
   mdsScaleTimer = null;
   // shut down the client/server IPC
   mdsStreamData (false);
   mdsPauseInProgress = false;
   var btn = document.getElementById("pause_btn");
   btn.value = "START";
   // append an informational to the notes
   var nodeNotes = document.getElementById("node_notes");
   var notesText = nodeNotes.innerHTML;
   nodeNotes.innerHTML = null;
   nodeNotes.innerHTML = mdsPauseStopText + notesText;
}

// make the first visible the second not
// -------------------------------------

function mdsForeFront (which1,which2)
{
   document.getElementById(which1).style.display = "block";
   document.getElementById(which2).style.display = "none";
}

// tell script to start/stop top-process monitoring
// ------------------------------------------------

function controlTopProc () {

   if (!ws) {
      // not using WebSocket so just restart the stream
      if (mdsTopProcVisible) mdsStreamData (true);
      return;
   }

   try {
      if (mdsTopProcVisible)
         ws.send ("topproc=1");
      else
         ws.send ("topproc=0");
   }
   catch (err) {
      // stop/start? WebSocket probably in process of being established
      setTimeout('controlTopProc()',1000);
   }

   return false;
}

// tell script to start/stop per-process monitoring
// ------------------------------------------------

function controlPerProc () {

   if (!ws) {
      // not using WebSocket so just restart the stream
      if (mdsPerProcPid) mdsStreamData (true);
      return;
   }

   try {
      ws.send ("perproc="+mdsPerProcPid);
   }
   catch (err) {
      // stop/start? WebSocket probably in process of being established
      setTimeout('controlPerProc()',1000);
   }

   return false;
}

// printing - a juggling act
// -------------------------

var mdsPrintBgCol;
var mdsPrintClickTimer = null;

function mdsPrintClick ()
{
   if (mdsPrintInProgress)
      mdsAfterPrint();
   else
   {
      mdsBeforePrint();
      setTimeout("window.print()",100);
   }
   return true;
}

function mdsBeforePrint ()
{
   if (mdsPrintInProgress) return true;
   mdsPrintInProgress = true;
   var btn = document.getElementById("print_btn");
   mdsPrintBgCol = btn.style.backgroundColor;
   btn.style.backgroundColor = mdsObviousBtn;
   clearTimeout(mdsPrintClickTimer);
   mdsPrintClickTimer = null;
   mdsPrintClickTimer = setTimeout("mdsPrintClick()",mdsPrintTimeout*1000);
   mdsResetAlert(false);
   mdsMakePrintable(true);
   mdsUpdateDisplay = false;
   return true;
};

function mdsAfterPrint()
{
   if (!mdsPrintInProgress) return true;
   mdsPrintInProgress = false;
   var btn = document.getElementById("print_btn");
   btn.style.backgroundColor = mdsPrintBgCol;
   clearTimeout(mdsPrintClickTimer);
   mdsPrintClickTimer = null;
   mdsResetAlert(true);
   mdsMakePrintable(false);
   if (!mdsPauseInProgress) mdsUpdateDisplay = true;
   return true;
};

// this is in-line code
var mdsMatchMedia;
if (window.matchMedia)
{
    mdsMatchMedia = window.matchMedia('print');
    mdsMatchMedia.addListener(function(mql) {
       if (mql.matches)
          mdsBeforePrint();
       else
          mdsAfterPrint();
    });
}
// a la MSIE
window.onbeforeprint = mdsBeforePrint;
window.onafterprint = mdsAfterPrint;

// (re)set WZ printable flag and redraw
// ------------------------------------

function mdsMakePrintable (MakePrint)
{
   // "jg_fast" is a datum in WZ_JSGRAPHICS.JS
   var jgFast = jg_fast;
   if (mdsWzPrintable = MakePrint)
   {
      // reset the WZ mode that garbles printable WZ graphics on MSIE
      if (mdsMSIEversion && jgFast) jg_fast = false;
   }
   for (which in mdsData)
   {
      if (which.substr(0,5) == "node_") mdsBarPlot(which);
      mdsHistPlot(which);
   }
   jg_fast = jgFast;
   mdsWzPrintable = false;
}

// return a string with the value as unit, kilo or mega
// ----------------------------------------------------

function mdsValueString (value)
{
   if (value > 999999999) {
      value = (value / 1073741824).toFixed(1);
      if (value > 99) value = (value/1).toFixed(0);
      var dvalue = value.toString() + "G";
   }
   else
   if (value > 999999) {
      value = (value / 1048576).toFixed(1);
      if (value > 99) value = (value/1).toFixed(0);
      var dvalue = value.toString() + "M";
   }
   else
   if (value > 9999) {
      value = (value / 1024).toFixed(1);
      if (value > 99) value = (value/1).toFixed(0);
      var dvalue = value.toString() + "k";
   }
   else
      var dvalue = value.toString();
   return dvalue;
}

// display a numeric value to the right of a bar or graph
// ------------------------------------------------------

function mdsDrawValue (jsgr,xpos,ypos,color,value)
{
   var dval = mdsValueString(value);
   xpos = xpos - 5 - (dval.length * 8) + (dval.length * 2);
   if (dval.indexOf("k") >= 0) xpos += 2;
   if (dval.indexOf(".") >= 0) xpos += 2;
   jsgr.setColor(color);
   jsgr.drawString(dval,xpos,ypos);
}

// initialise a single datum bar graph
// -----------------------------------

function mdsBarGraph (which)
{
   var jsgr = mdsData[which].bar_jsgr;
   jsgr.recreate();
   jsgr.setPrintable(mdsWzPrintable);
   jsgr.setFont ("arial","10px",Font.PLAIN); 
   if (mdsData[which].active)
      jsgr.setColor("white");
   else
      jsgr.setColor("#d3d3d3"); // "lightGray" - stupid MSIE
   jsgr.fillRect (0, 0, mdsBarWidth+2, mdsBarHeight);
}

// add the axes and title of a bar graph then paint
// ------------------------------------------------

function mdsBarPaint (which)
{
   var jsgr = mdsData[which].bar_jsgr;
   jsgr.setColor("black");
   jsgr.drawRect (0, 0, mdsBarWidth+2, mdsBarHeight);
   if (which == "cpu" && mdsCpuCount > 1)
      jsgr.drawString(mdsData[which].title+"x"+mdsCpuCount,7,2);
   else
      jsgr.drawString(mdsData[which].title,7,2);
   jsgr.paint();
}

// render a single datum of the bar graph
// --------------------------------------

function mdsBarPlot (which)
{
   if (!mdsUpdateDisplay) return;

   if (mdsData[which] == undefined) return;

   mdsBarGraph(which);
   if (!mdsData[which].active)
   {
      mdsBarPaint(which);
      return;
   }

   var height = mdsBarHeight;

   var jsgr = mdsData[which].bar_jsgr;
   jsgr.setColor("white");
   jsgr.fillRect (0, 0, mdsBarWidth+2, mdsBarHeight+2);

   mdsDrawValue(jsgr,mdsBarWidth,2,"black",mdsData[which].scale);

   var widthAve = mdsData[which].ave *
                  mdsBarWidth / mdsData[which].scale;
   if (widthAve > mdsBarWidth) widthAve = mdsBarWidth;
   var widthVal1 = mdsData[which].val1 *
                   mdsBarWidth / mdsData[which].scale;
   if (widthVal1 > mdsBarWidth) widthVal1 = mdsBarWidth;

   if (mdsData[which].val2 >= 0)
      var widthVal2 = mdsData[which].val2 *
                      mdsBarWidth / mdsData[which].scale;
   else
      var widthVal2 = 0;

   if (widthAve > 0 && widthAve > widthVal1 && widthAve > widthVal2)
   {
      jsgr.setColor(mdsBarColAve);
      jsgr.fillRect (1, 0, widthAve+1, height);
      if (widthAve > 60)
         mdsDrawValue(jsgr,widthAve,2,"black",mdsData[which].ave);
   }

   if (widthVal1 > 0 && widthVal1 > widthVal2)
   {
      jsgr.setColor(mdsBarCol1);
      jsgr.fillRect (1, 0, widthVal1+1, height);
      if (widthVal1 > 60)
         mdsDrawValue(jsgr,widthVal1,2,"black",mdsData[which].val1);
   }

   if (widthVal2 > 0)
   {
      jsgr.setColor(mdsBarCol2);
      jsgr.fillRect (1, 0, widthVal2+1, height);
      if (widthVal2 > 60)
         mdsDrawValue(jsgr,widthVal2,2,"black",mdsData[which].val2);
   }

   mdsBarPaint(which);
}

// initialise a history line graph
// -------------------------------

function mdsHistGraph (which)
{
   var jsgr = mdsData[which].hist_jsgr;
   jsgr.recreate();
   jsgr.setPrintable(mdsWzPrintable);
   jsgr.setFont ("arial","10px",Font.PLAIN); 
   if (mdsData[which].hist1 != undefined)
      if (mdsData[which].active)
         jsgr.setColor("white");
      else
         jsgr.setColor("#d3d3d3"); // "lightGray" - stupid MSIE
   jsgr.fillRect (1, 1, mdsGraphWidth+1, mdsGraphHeight+1);
}

// provide the axes and title of the graph then display
// ----------------------------------------------------

function mdsHistPaint (which)
{
   var jsgr = mdsData[which].hist_jsgr;
   jsgr.setColor("black");
   jsgr.drawRect (0, 0, mdsGraphWidth+2, mdsGraphHeight+2);
   for (var x = 30; x < mdsGraphWidth-2; x += 30)
   {
      if (x % 60)
         jsgr.drawLine (x+1, mdsGraphHeight+1, x+1, mdsGraphHeight+1-2);
      else
         jsgr.drawLine (x+1, mdsGraphHeight+1, x+1, mdsGraphHeight+1-5);
   }
   if ((which == "node_cpu" || which == "proc_cpu") && mdsCpuCount > 1)
      jsgr.drawString(mdsData[which].title+"x"+mdsCpuCount,5,5);
   else
      jsgr.drawString(mdsData[which].title,5,5);
   jsgr.paint();
}

// display the numeric scale and values on the plot
// ------------------------------------------------

function mdsHistScale (jsgr,which,value1,value2,scale)
{
   if (scale >= 0)
      mdsDrawValue(jsgr,mdsGraphWidth,5,"black",scale);
   if (scale > 0 && value1 >= 0)
      mdsDrawValue(jsgr,mdsGraphWidth,20,mdsHistCol1,value1);
   if (scale > 0 && value1 > 0 && value2 >= 0)
      mdsDrawValue(jsgr,mdsGraphWidth,35,mdsHistCol2,value2);
}

// display an average on the plot
// ------------------------------

function mdsHistAverage (jsgr,value,scale)
{
   if (value <= 0 || value == NaN) return;
   var average = value * mdsGraphHeight / scale;
   if (average <= 0) return;
   if (average > mdsGraphHeight) average = mdsGraphHeight;
   jsgr.setColor(mdsHistColAve);
   jsgr.drawLine (mdsGraphWidth+1, mdsGraphHeight-average+1, 1,
                  mdsGraphHeight-average+1);
   if (average < 12)
      jsgr.drawString(mdsValueString(value),5,mdsGraphHeight-15-average+2);
   else
   if (average < 85)
      jsgr.drawString(mdsValueString(value),5,mdsGraphHeight-average+2);
   else
      jsgr.drawString(mdsValueString(value),35,mdsGraphHeight-average+2);
}

// plot a history graph from stored sparse data
// --------------------------------------------

function mdsHistPlot (which)
{
   if (!mdsUpdateDisplay) return;

   if (!mdsData[which].hist1) return;

   if ((!mdsNodeHistVisible && which.substr(0,5) == "node_") ||
       (!mdsPerProcHistVisible && which.substr(0,5) == "proc_"))
   {
      // obviously do not plot if not visible!
      if (mdsData[which].visible)
      {
         mdsData[which].visible = false;
         mdsData[which].hist_jsgr.recreate();
         mdsData[which].hist_jsgr.setPrintable(mdsWzPrintable);
      }
      return;
   }
   mdsData[which].visible = true;

   mdsHistGraph(which);

   if (!mdsData[which].active)
   {
      mdsHistPaint (which);
      return;
   }

   var jsgr = mdsData[which].hist_jsgr;

   // wz canvas begins in the top-left, pen beneath and right of coords
   var scale = mdsData[which].scale;

   mdsHistAverage(jsgr,mdsData[which].ave,scale);

   var scale = mdsData[which].scale;

   var hist1 = mdsData[which].hist1,
       hist2 = mdsData[which].hist2;

   var x = y = lastx1 = lasty1 = lastx2 = lasty2 = 0;
   for (tdata in hist1)
   {
      x = mdsTimeStamp - parseInt(tdata);

      // eliminate stale data
      if (x > mdsGraphWidth)
      {
         if (x > mdsGraphWidth + 10)
         {
            delete hist1[tdata];
            delete hist2[tdata];
            continue;
         }
         x = mdsGraphWidth;
      }

      y = hist1[tdata] * mdsGraphHeight / scale;
      if (y > mdsGraphHeight) y = mdsGraphHeight;
      jsgr.setColor(mdsHistCol1);
      if (lasty1 > 0)
         jsgr.drawLine (mdsGraphWidth-lastx1, 100-lasty1+1,
                        mdsGraphWidth-x, 100-lasty1+1);
      jsgr.drawLine (mdsGraphWidth-x, 100-lasty1+1,
                     mdsGraphWidth-x, 100-y+1);
      lastx1 = x;
      lasty1 = y;

      if (hist2[tdata] != undefined)
      {
         y = hist2[tdata] * mdsGraphHeight / scale;
         if (y > mdsGraphHeight) y = mdsGraphHeight;
         jsgr.setColor(mdsHistCol2);
         if (lasty2 > 0)
            jsgr.drawLine (mdsGraphWidth-lastx2, 100-lasty2+1,
                           mdsGraphWidth-x, 100-lasty2+1);
         jsgr.drawLine (mdsGraphWidth-x, 100-lasty2+1,
                        mdsGraphWidth-x, 100-y+1);
         lastx2 = x;
         lasty2 = y;
      }
   }

   if (lasty1 > 0)
   {
      jsgr.setColor(mdsHistCol1);
      jsgr.drawLine (mdsGraphWidth-lastx1+1, 100-lasty1+1,
                     mdsGraphWidth+1, 100-lasty1+1);
   }
   if (lasty2 > 0)
   {
      jsgr.setColor(mdsHistCol2);
      jsgr.drawLine (mdsGraphWidth-lastx2+1, 100-lasty2+1,
                     mdsGraphWidth+1, 100-lasty2+1);
   }

   mdsHistScale(jsgr,which,mdsData[which].val1,
                           mdsData[which].val2,
                           mdsData[which].scale);

   mdsHistPaint (which);
}

// WebSocket support (for WASD v10.1ff)
// ------------------------------------

var ws = null;
var nodeControl = null;

function webSocketOpen () {

   if (!("WebSocket" in window)) return false;

   if (ws) return true;

   if (window.location.protocol == "http:")
      var URL = "ws://";
   else
      var URL = "wss://";
   URL += window.location.host + mdsScriptName + "?" +
          (mdsTopProcVisible ? "1" : "0") + "+" + mdsPerProcPid;

   ws = new WebSocket(URL);

   ws.onopen = function(evt) {
   };

   ws.onclose = function(evt) {
      if (ws) ws.close();
      ws = null;
      // try to reconnect in a staggered fashion (in case multiple CGIplus)
      var tout = 1500 + Math.floor((Math.random()*1000));
      if (mdsRunning) setTimeout ("webSocketOpen()", tout);
  };

   ws.onmessage = function (evt) { 
      var data = evt.data;
      eval (data);
   };

   return true;
}

function webSocketClose () {

   if (!WASDwebSocket) return;
   if (!("WebSocket" in window)) return;

   if (ws) ws.close();
   ws = null;
}

// fin!
